package apps;

import java.io.File;
import java.io.IOException;

import com.ibm.rhapsody.apps.io.RAStorage;
import com.ibm.rhapsody.apps.utils.Reporter;

public class MBDGConfiguration {
	public static String DocumentTemplatePath = "${OMROOT}/Profiles/ModelBasedDocumentGeneration/templates/document.dta";
	public static String ContentDocumentTemplatePath = "${OMROOT}/Profiles/ModelBasedDocumentGeneration/templates/content.dta";
	public static String RPELibrariesDir = "${OMROOT}/Profiles/ModelBasedDocumentGeneration/libs";
	public static String RPEProductHome = RPELibrariesDir+"/productHome/home";
	
	private String documentPath = "";
	private String tempDirectory = "";
	private String documentSpecificationPath = "";
	private String documentSpecificationsDirectory = "";
	private String templatesDirectory = "";
	private String generatedTemplatePath = "";
	private String docgenPath = "";
	private String debugMode = "";

	public MBDGConfiguration clone() {
		MBDGConfiguration configuration = new MBDGConfiguration();
		configuration.setDocumentPath(this.getDocumentPath());
		configuration.setTempDirectory(this.getTempDirectory());
		configuration.setDocumentSpecificationPath(this.getDocumentSpecificationPath());
		configuration.setDocumentSpecificationsDirectory(this.getDocumentSpecificationsDirectory());
		configuration.setTemplatesDirectory(this.getTemplatesDirectory());
		configuration.setGeneratedTemplatePath(this.getGeneratedTemplatePath());
		configuration.setDocgenPath(this.getDocgenPath());
		configuration.setDebugMode(this.getDebugMode());
		return configuration;
	}	
	
	public boolean equals(MBDGConfiguration configuration) {
		boolean res = true;
		res = this.getDocumentPath().equals(configuration.getDocumentPath()) &&
			  this.getDocumentSpecificationPath().equals(configuration.getDocumentSpecificationPath()) && 
			  this.getTempDirectory().equals(configuration.getTempDirectory()) &&
			  this.getDocumentSpecificationsDirectory().equals(configuration.getDocumentSpecificationsDirectory()) &&
			  this.getTemplatesDirectory().equals(configuration.getTemplatesDirectory()) &&
			  this.getGeneratedTemplatePath().equals(configuration.getGeneratedTemplatePath()) &&
			  this.getDocgenPath().equals(configuration.getDocgenPath()) &&
			  this.getDebugMode().equals(configuration.getDebugMode());
		return res;
	}
	
    public void save(String path) {
   		MBDGExpander expander = new MBDGExpander();
   		setDocumentPath(expander.collapsePath(getDocumentPath()));
   		setTempDirectory(expander.collapsePath(getTempDirectory()));
   		setDocumentSpecificationPath(expander.collapsePath(getDocumentSpecificationPath()));
   		setDocumentSpecificationsDirectory(expander.collapsePath(getDocumentSpecificationsDirectory()));
   		setTemplatesDirectory(expander.collapsePath(getTemplatesDirectory()));
   		setGeneratedTemplatePath(expander.collapsePath(getGeneratedTemplatePath()));
   		setDocgenPath((expander.collapsePath(getDocgenPath())));
   		
   		if (!MBDGExpander.getTempDir().equals("")) {
   			setTempDirectory("${TEMP_DIR}");
   		}
   		
   		String configPath = expander.expand(path);
   		RAStorage storage = new RAStorage();
   		storage.save(configPath, this);
	}	
	
    public void load(String path) {
   		String configPath = new MBDGExpander().expand(path);
   		RAStorage storage = new RAStorage();
   		storage.load(configPath, this);
   		
   		MBDGExpander expander = new MBDGExpander();
   		if (getTempDirectory().equals("${TEMP_DIR}")) {
   			try {
				MBDGExpander.setTempDir(MBDGUtils.createTempDirectory("DocGen"));
			} catch (IOException e) {
				Reporter.report(e);
			}
   		}
		setDocumentPath(expander.expand(getDocumentPath()));
		setTempDirectory(expander.expand(getTempDirectory()));
		setDocumentSpecificationPath(expander.expand(getDocumentSpecificationPath()));
		setDocumentSpecificationsDirectory(expander.expand(getDocumentSpecificationsDirectory()));
   		setTemplatesDirectory(expander.expand(getTemplatesDirectory()));
   		setGeneratedTemplatePath(expander.expand(getGeneratedTemplatePath()));
   		setDocgenPath((expander.expand(getDocgenPath())));
	}
    
    public String getRPELibrariesDir() {
    	MBDGExpander expander = new MBDGExpander();
    	return expander.expand(RPELibrariesDir);
    }
    
    public String getRPEProductHome() {
    	MBDGExpander expander = new MBDGExpander();
    	return expander.expand(RPEProductHome);
    }    
    
	public String getWorkareaDirectory() {
		return getTempDirectory()+File.separator+"/workarea";
	}
	
	public String getGeneratedTemplateDirectory() {
		return getWorkareaDirectory()+File.separator+"/generated"; 
	}	
	
	public File getGeneratedTemplateXmlFile() {
		String dir = getGeneratedTemplateDirectory();
		return new File(dir+File.separator+"template.xml");
	}	
	
	public String getTemplateUnzippedDirectory(String path) {
		File file = new File(path);
		String name = file.getName().replace(".dta", "");
		return getTempDirectory()+File.separator+name;
	}
	
	public File getTemplateXmlFile(File templateFile) {
		String name = templateFile.getName().replace(".dta", "");
		return new File(getTempDirectory()+File.separator+name+File.separator+"template.xml");
	}		
	
	public String getTemplateXmlPath(String templatePath) {
		return getTemplateXmlFile(new File(templatePath)).getAbsolutePath();
	}    
    
	public String getDocumentPath() {
   		return documentPath;
	}
	
	public void setDocumentPath(String documentPath) {
   		this.documentPath = documentPath;		
	}    

	public String getDocumentSpecificationPath() {
		return documentSpecificationPath;
	}

	public void setDocumentSpecificationPath(String documentSpecificationPath) {
		this.documentSpecificationPath = documentSpecificationPath;
	}	
	
	public String getTempDirectory() {
		return tempDirectory;
	}

	public void setTempDirectory(String tempDirectory) {
		this.tempDirectory = tempDirectory;
	}

	public String getDocumentSpecificationsDirectory() {
		return documentSpecificationsDirectory;
	}

	public void setDocumentSpecificationsDirectory(
			String documentSpecificationsDirectory) {
		this.documentSpecificationsDirectory = documentSpecificationsDirectory;
	}

	public String getTemplatesDirectory() {
		return templatesDirectory;
	}

	public void setTemplatesDirectory(String templatesDirectory) {
		this.templatesDirectory = templatesDirectory;
	}

	public String getGeneratedTemplatePath() {
		return generatedTemplatePath;
	}

	public void setGeneratedTemplatePath(String generatedTemplatePath) {
		this.generatedTemplatePath = generatedTemplatePath;
	}

	public String getDocgenPath() {
		return docgenPath;
	}

	public void setDocgenPath(String docgenPath) {
		this.docgenPath = docgenPath;
	}

	public String getDebugMode() {
		return debugMode;
	}
	
	public boolean isDebugMode() {
		return Boolean.valueOf(debugMode);
	}	

	public void setDebugMode(String debugMode) {
		this.debugMode = debugMode;
	}
	
	public void setDebugMode(boolean isDebugMode) {
		this.debugMode = String.valueOf(isDebugMode);
	}		
}
