package apps;

import java.util.ArrayList;
import java.util.List;

import com.telelogic.rhapsody.core.IRPInstance;
import com.telelogic.rhapsody.core.IRPModelElement;

public class MBDGElement {
	private IRPModelElement element;
	private List<MBDGElement> descendants;
	private IRPModelElement referencedModelElement;
	private IRPModelElement scope;
	private List<IRPModelElement> items;
	private boolean isRoot;
	private StringBuilder template;
	private MBDGElement owner;
	private String elementHandle;
	private String elementContext;
	private String elementQuery;
	private String replaceableContext;
	
	public MBDGElement(IRPModelElement modelElement) {
		init(modelElement);
	}
	
	public MBDGElement(IRPInstance part) {
		if (part != null) {
			init(part.getOtherClass());
		}
	}	

	public MBDGElement() {
		init(null);
	}

	public MBDGElement(MBDGElement element) {
		init(element.getElement());
	}
	
	public MBDGElement partialClone() {
		MBDGElement clone = new MBDGElement();
		clone.setElement(element);
		clone.setDescendants(new ArrayList<MBDGElement>());
		clone.setReferencedModelElement(null);
		clone.setScope(scope);
		clone.setItems(null);
		clone.setRoot(isRoot);
		clone.setTemplate(template);
		clone.setOwner(null);
		clone.setElementHandle("");
		clone.setElementContext("");
		clone.setElementQuery("");		
		return clone;
	}	

	private void init(IRPModelElement element) {
		setElement(element);
		setDescendants(new ArrayList<MBDGElement>());
		setReferencedModelElement(null);
		setScope(null);
		setItems(new ArrayList<IRPModelElement>());
		setRoot(false);
		setTemplate(new StringBuilder());
		setOwner(null);
		setElementHandle("");
		setElementContext("");
		setElementQuery("");	
	}
	
	@SuppressWarnings("unchecked")
	public List<IRPModelElement> getNestedElements() {
		return element.getNestedElements().toList();
	}
	
	public void addDescendent(MBDGElement descendant) {
		descendants.add(descendant);
		descendant.setOwner(this);
	}
	
	public IRPModelElement getElement() {
		return element;
	}

	public void setElement(IRPModelElement element) {
		this.element = element;
	}
	
	public List<MBDGElement> getDescendants() {
		return descendants;
	}

	public void setDescendants(List<MBDGElement> descendants) {
		this.descendants = descendants;
	}
	
	public IRPModelElement getReferencedModelElement() {
		return referencedModelElement;
	}

	public void setReferencedModelElement(IRPModelElement referencedModelElement) {
		this.referencedModelElement = referencedModelElement;
	}	

	public IRPModelElement getScope() {
		return scope;
	}

	public void setScope(IRPModelElement scope) {
		this.scope = scope;
	}
	
	public void initScope(MBDGElement ownerElement) {
		if (ownerElement.getReferencedModelElement() != null) {
			setScope(ownerElement.getReferencedModelElement());
		} else {
			setScope(ownerElement.getScope());
		}
		
	}	
	
	public List<IRPModelElement> getItems() {
		return items;
	}

	public void setItems(List<IRPModelElement> items) {
		this.items = items;
	}	
	
	public String toString() {
		String name = (getElement() != null ? getElement().getDisplayName() : "");
		String type = (getElement() != null ? getElement().getUserDefinedMetaClass() : "");
		String referenceModelElementName = (referencedModelElement != null ? referencedModelElement.getName() : "");
		String scopeName = (scope != null ? scope.getName() : "");
		String ownerName = ((owner != null && owner.getElement() != null) ? owner.getElement().getDisplayName() : "");

		StringBuilder itemsStr = new StringBuilder();
		if (items != null) {
			for (IRPModelElement item : items) {
				itemsStr.append(item.getName() + ", ");
			}
		}

		String res= String.format("%s [%s] Template: handle = (%s)  context = (%s) query = (%s)  ownerHandle = (%s) Data: reference = (%s)  owner = (%s)  Collection: scope = (%s) items = (%s)", name, type, getElementHandle(), getElementContext(), getElementQuery(), getOwnerElementHandle(), referenceModelElementName, ownerName, scopeName, itemsStr);
		return res;
	}

	public boolean isRoot() {
		return isRoot;
	}

	public void setRoot(boolean isRoot) {
		this.isRoot = isRoot;
	}

	public StringBuilder getTemplate() {
		return template;
	}

	public void setTemplate(StringBuilder template) {
		this.template = template;
	}

	public MBDGElement getOwner() {
		return owner;
	}

	public void setOwner(MBDGElement owner) {
		this.owner = owner;
	}

	public String getOwnerElementHandle() {
		String handle = "";
		MBDGElement tmpOwner = owner;
		while (tmpOwner != null && handle.isEmpty()) {
			handle = tmpOwner.getElementHandle();
			tmpOwner = tmpOwner.getOwner();
		}
		return handle;
	}

	public String getElementHandle() {
		return elementHandle;
	}

	public void setElementHandle(String elementHandle) {
		this.elementHandle = elementHandle;
	}

	public String getElementContext() {
		return elementContext;
	}

	public void setElementContext(String elementContext) {
		this.elementContext = elementContext;
	}

	public String getElementQuery() {
		return elementQuery;
	}

	public void setElementQuery(String elementQuery) {
		this.elementQuery = elementQuery;
	}

	public String getReplaceableContext() {
		return replaceableContext;
	}

	public void setReplaceableContext(String replaceableContext) {
		this.replaceableContext = replaceableContext;
	}		
}
