package apps;

import java.util.HashMap;
import java.util.Map;

public class MBDGTemplateHandlesIdsUpdater {
	private static final String HANDLE_PATTERN = "handle=\"$";
	private static final String CONTEXT_PATTERN = "context=\"$";
	private static final String ID_PATTERN = "element id=\"";
	
	private static int globalHandle = 1;
	private static int globalId = 1;
	private Map<String, String> handlesMap;
	private Map<String, String> idsMap;
	private StringBuilder template;
	
	public MBDGTemplateHandlesIdsUpdater(StringBuilder template) {
		this.template = template;
		handlesMap = new HashMap<String, String>();
		idsMap = new HashMap<String, String>();
	}
	
	public static void reset() {
		globalHandle = 1;
		globalId = 1;
	}

	public void update() {
		mapHandles();
		replaceHandles();
		mapIds();
		replaceIds();
	}

	private void mapHandles() {
		int index = 0;
		while(index != -1) {
			index = template.indexOf(HANDLE_PATTERN, index);
			if (index != -1) {
				String handle = getHandle(index);
				mapHandle(handle);
				index++;
			}
		} 		
	}

	private String getHandle(int start) {
		String handle = getHandle(start, HANDLE_PATTERN);
		if (!handle.startsWith("$")) {
			handle = getHandle(start, CONTEXT_PATTERN);
		}
		return handle;
	}	

	private String getHandle(int start, String pattern) {
		String handle = null;
		if (start > 0) {
			int length = pattern.length();
			int end = template.indexOf("\"", start+length);
			handle = template.substring(start+length-1, end);
		}
		return handle;
	}
	

	private void mapHandle(String handle) {
		String newHandle = handlesMap.get(handle);
		if (newHandle == null) {
			newHandle = String.format("$%s", globalHandle++);
			handlesMap.put(handle, newHandle);
		}
	}	

	private void replaceHandles() {
		int index = 0;
		while(index != -1) {
			index = findHandleFromIndex(index);
			if (index != -1) {
				String handle = getHandle(index);
				replaceHandle(index, handle);
				index++;
			}
		} 
	}

	private int findHandleFromIndex(int index) {
		int index1 = template.indexOf(HANDLE_PATTERN, index);
		int index2 = template.indexOf(CONTEXT_PATTERN, index);
		int res = Math.min(index1, index2);
		if (res == -1) {
			res = Math.max(index1, index2);
		}
		return res;
	}	

	private void replaceHandle(int start, String handle) {
		String newHandle = handlesMap.get(handle);
		if (start > 0 && newHandle != null) {
			start = template.indexOf(handle, start);
			int end = start+handle.length();
			template.replace(start, end, newHandle);
		}
	}
	
	private void mapIds() {
		int index = 0;
		while(index != -1) {
			index = template.indexOf(ID_PATTERN, index);
			if (index != -1) {
				String id = getId(index);
				mapId(id);
				index++;
			}
		} 
	}	
	
	private void replaceIds() {
		int index = 0;
		while(index != -1) {
			index = template.indexOf(ID_PATTERN, index);
			if (index != -1) {
				String id = getId(index);
				replaceId(index, id);
				index++;
			}
		} 
	}
	
	private String getId(int start) {
		String id = null;
		if (start > 0) {
			int length = ID_PATTERN.length();
			int end = template.indexOf("\"", start+length);
			id = template.substring(start+length, end);
		}
		return id;
	}
	
	private void mapId(String id) {
		String newId = idsMap.get(id);
		if (newId == null) {
			newId = String.format("%s", globalId++);
			idsMap.put(id, newId);
		}
	}	
	
	private void replaceId(int start, String id) {
		String newId = idsMap.get(id);
		if (start > 0 && newId != null) {
			start = template.indexOf(id, start);
			int end = start+id.length();
			template.replace(start, end, newId);
		}
	}	
}
