package com.ibm.rhapsody.sysML.proxyportWizards;

import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.HashMap;

import org.eclipse.swt.graphics.Point;

import com.telelogic.rhapsody.core.IRPApplication;
import com.telelogic.rhapsody.core.IRPAssociationClass;
import com.telelogic.rhapsody.core.IRPCollection;
import com.telelogic.rhapsody.core.IRPDiagram;
import com.telelogic.rhapsody.core.IRPFlow;
import com.telelogic.rhapsody.core.IRPGraphEdge;
import com.telelogic.rhapsody.core.IRPGraphNode;
import com.telelogic.rhapsody.core.IRPGraphicalProperty;
import com.telelogic.rhapsody.core.IRPInstanceValue;
import com.telelogic.rhapsody.core.IRPModelElement;
import com.telelogic.rhapsody.core.IRPTag;
import com.telelogic.rhapsody.core.IRPValueSpecification;

public class MappingGraphicsCreator {
	private IRPAssociationClass mappingAssocClass = null;
	private IRPApplication rhapsody = null;
	
	public MappingGraphicsCreator(IRPAssociationClass MappingGraphicsCreator, IRPApplication rhapsody) {
		this.mappingAssocClass = MappingGraphicsCreator;
		this.rhapsody = rhapsody;
	}
	
	
	public void createGraphics() {
		if(mappingAssocClass != null && rhapsody != null) {
			IRPCollection flows = mappingAssocClass.getFlows();
			if(flows != null && flows.getCount() > 0) {
				String ibdName = Utils.getUniqueName(mappingAssocClass, "Mapping", "Internal Block Diagram");
				if(ibdName != null) {
					IRPDiagram ibd = (IRPDiagram) mappingAssocClass.addNewAggr("Internal Block Diagram", ibdName);
					if(ibd != null) {
						ibd.setPropertyValue("ObjectModelGe.Class.ShowPorts", "False");
						ibd.setPropertyValue("ObjectModelGe.Interface.ShowPorts", "False");
						ibd.setPropertyValue("ObjectModelGe.Object.ShowPorts", "False");
						ibd.setPropertyValue("General.Graphics.ContextByTopology", "Attribute,FlowProperty");
						ibd.setPropertyValue("ObjectModelGe.Flow.flowKeyword", "False");
						ibd.addProperty("ObjectModelGe.Port.CreatePortAsBox", "Bool", "True");
												
						//first create the graphical nodes
						createMappingGraphicalElements(ibd, flows, true);
						createMappingGraphicalElements(ibd, flows, false);
					
						//next create the binding connector of the mapping
						if(flows != null && flows.getCount() > 0) {
							for(int i = 1 ; i <= flows.getCount() ; i++) {
								IRPFlow flow = (IRPFlow)flows.getItem(i);
								IRPModelElement end1 = flow.getEnd1();
								if(flow.getEnd1Port() != null)
									end1 = flow.getEnd1Port();
								IRPModelElement end2 = flow.getEnd2();
								if(flow.getEnd2Port() != null)
									end2 = flow.getEnd2Port();
								
								if(end1 != null && end2 != null) {
									IRPTag contextTag = flow.getTag("SourceContext");
									IRPCollection context = null;	
									if(contextTag != null) 
										context = contextTag.getValueSpecifications();
									IRPGraphNode end1Node = Utils.getGraphicalNode(ibd, end1, context);
									
									contextTag = flow.getTag("TargetContext");
									if(contextTag != null) 
										context = contextTag.getValueSpecifications();
									IRPGraphNode end2Node = Utils.getGraphicalNode(ibd, end2, context);
									if(end1Node != null && end2Node != null) {
										Point ptNewSrc = Utils.getMiddlePointOfGraphicalNode(end1Node, false);
										Point ptNewTrg = Utils.getMiddlePointOfGraphicalNode(end2Node, false);
										if(ptNewSrc != null && ptNewTrg != null) {
											IRPGraphEdge edge = ibd.addNewEdgeForElement(flow, end1Node, ptNewSrc.x, ptNewSrc.y, 
													end2Node, ptNewTrg.x, ptNewTrg.y);	
											if(edge != null) {
												edge.setGraphicalProperty("LineStyle", "Straight");
											}
										}
									}
								}
							}
						}
						ibd.removeProperty("ObjectModelGe.Port.CreatePortAsBox");
					}
				}
			}
		}
	}

	private void createMappingGraphicalElements(IRPDiagram ibd, IRPCollection flows, boolean bSourceSide) {
		if(ibd != null && flows != null) {
			Map<IRPModelElement, List<IRPModelElement>> childrenMap = buildChildrenMapFromMapping(flows, bSourceSide);
			Map<String,Point> sizeMap = buildSizeMap(flows, childrenMap, bSourceSide);
			if(childrenMap != null && sizeMap != null) {				
				if(flows != null && flows.getCount() > 0) {
					for(int i = 1 ; i <= flows.getCount() ; i++) {
						IRPFlow flow = (IRPFlow)flows.getItem(i);
						IRPTag contextTag = null;
						if(bSourceSide)
							contextTag = flow.getTag("SourceContext");
						else
							contextTag = flow.getTag("TargetContext");
						
						if(contextTag != null) {
							IRPCollection context = contextTag.getValueSpecifications();
							if(context != null && context.getCount() > 0) {
								for(int j = 1 ; j <= context.getCount() ; j++) {
									IRPValueSpecification vs = (IRPValueSpecification) context.getItem(j);
									if(vs instanceof IRPInstanceValue) {
										IRPInstanceValue iv = (IRPInstanceValue)vs;
										IRPModelElement elem = iv.getValue();
										IRPGraphNode parentNode = Utils.getGraphicalNode(ibd, elem, null, null);
										if(Utils.getStereotype(elem, "ParticipantProperty") != null) {
											if(parentNode == null) {
												Point size = sizeMap.get(elem.getName());
												if(size == null)
													continue;
				
												int width = size.x;
												int height = size.y;
												int xPosition = 50;
												int yPosition = 50;
												if(!bSourceSide)
													xPosition += width + 150;
												parentNode = ibd.addNewNodeForElement(elem, xPosition, yPosition, width, height);	
												parentNode.setGraphicalProperty("StructureView", "True");
											}
											else
												continue;
										}
										List<IRPModelElement> childrenList = childrenMap.get(elem);
										if(parentNode != null && childrenList != null && childrenList.size() > 0) {
											Point ptParentTopLeft = null;
											int parentWidth = 0;
											int parentHeight = 0;
											IRPGraphicalProperty prp = parentNode.getGraphicalProperty("Position");
											if(prp != null) {
												ptParentTopLeft = Utils.extractPoint(prp.getValue());
											}
											prp = parentNode.getGraphicalProperty("Width");
											if(prp != null && prp.getValue() != null) {
												parentWidth = Integer.parseInt(prp.getValue());
											}
											prp = parentNode.getGraphicalProperty("Height");
											if(prp != null && prp.getValue() != null) {
												parentHeight = Integer.parseInt(prp.getValue());
											}
											
											if(ptParentTopLeft != null && parentWidth > 0 && parentHeight > 0) {
												int xMargin = 20;
												int yMargin = 20;
												int lastYPosition = ptParentTopLeft.y + yMargin;
												int childXPosition = ptParentTopLeft.x + xMargin;
												
												for(IRPModelElement child : childrenList) {
													IRPGraphNode childNode = Utils.getGraphicalNode(ibd, child, parentNode, null);
													if(childNode != null)
														continue;
													
													Point size = sizeMap.get(getFullContextName(child, context, elem));
													if(size == null)
														continue;
													
													int childWidth = size.x;
													int childHeight = size.y;
													int childYPosition = lastYPosition + yMargin;
													childNode = ibd.addNewNodeForElement(child, childXPosition, childYPosition, childWidth, childHeight);
													childNode.setGraphicalProperty("StructureView", "True");
													lastYPosition = childYPosition + childHeight;
												}
											}
										}
									}
								}
							}
						}
					}
				}
			}
		}
	}
		
	private Map<IRPModelElement, List<IRPModelElement>> buildChildrenMapFromMapping(IRPCollection flows, boolean bSourceContext) {
		Map<IRPModelElement, List<IRPModelElement>> childrenMap = null;
		if(flows != null && flows.getCount() > 0) {
			for(int i = 1 ; i <= flows.getCount() ; i++) {
				IRPFlow flow = (IRPFlow)flows.getItem(i);
				IRPTag contextTag = null;
				if(bSourceContext)
					contextTag = flow.getTag("SourceContext");
				else
					contextTag = flow.getTag("TargetContext");
				
				if(contextTag != null) {
					IRPCollection context = contextTag.getValueSpecifications();
					if(context != null && context.getCount() > 0) {
						if(childrenMap == null)
							childrenMap = new HashMap<IRPModelElement, List<IRPModelElement>>();
						IRPModelElement previousParent = null;
						for(int j = 1 ; j <= context.getCount() ; j++) {
							IRPValueSpecification vs = (IRPValueSpecification) context.getItem(j);
							if(vs instanceof IRPInstanceValue) {
								IRPInstanceValue iv = (IRPInstanceValue)vs;
								IRPModelElement elem = iv.getValue();
								if(elem != null) {
									if(previousParent != null) {
										List<IRPModelElement> childrenList = childrenMap.get(previousParent);
										if(childrenList != null && childrenList.contains(elem) == false) 
											childrenList.add(elem);
									}
									
									if(childrenMap.get(elem) == null) {
										List<IRPModelElement> childrenList = new LinkedList<IRPModelElement>();
										childrenMap.put(elem, childrenList);
									}
									previousParent = elem;
									if(j == context.getCount()) {
										IRPModelElement mappingElement = null;
										if(bSourceContext) {
											mappingElement = flow.getEnd1();
											if(flow.getEnd1Port() != null)
												mappingElement = flow.getEnd1Port();
										}
										else {
											mappingElement = flow.getEnd2();
											if(flow.getEnd2Port() != null)
												mappingElement = flow.getEnd2Port();
										}
										List<IRPModelElement> childrenList = childrenMap.get(elem);
										if(childrenList != null && mappingElement != null && childrenList.contains(mappingElement) == false) 
											childrenList.add(mappingElement);
									}
								}
							}
						}
					}
				}
			}
		}
		return childrenMap;
	}

	private Map<String,Point> buildSizeMap(IRPCollection flows, Map<IRPModelElement, List<IRPModelElement>> childrenMap,  boolean bSourceContext) {
		Map<String,Point> sizeMap = null;
		if(flows != null && flows.getCount() > 0 && childrenMap != null) {
			int i = 0;
			int leafWidth = 70;
			int leafHeight = 50;
			int xMargin = 20;
			int yMargin = 20;
			int nContextDpeth = 0;
			
			//first build the leaf element size, and find the maximum context depth
			for(i = 1 ; i <= flows.getCount() ; i++) {
				IRPFlow flow = (IRPFlow)flows.getItem(i);
				IRPModelElement end = null;
				if(bSourceContext) {
					end = flow.getEnd1();
					if(flow.getEnd1Port() != null)
						end = flow.getEnd1Port();
				}
				else {
					end = flow.getEnd2();
					if(flow.getEnd2Port() != null)
						end = flow.getEnd2Port();
				}
				
				IRPTag contextTag = null;
				if(bSourceContext)
					contextTag = flow.getTag("SourceContext");
				else
					contextTag = flow.getTag("TargetContext");
				
				if(contextTag != null) {
					IRPCollection context = contextTag.getValueSpecifications();
					if(context != null && context.getCount() > nContextDpeth)
						nContextDpeth = context.getCount();
					
					if(sizeMap == null)
						sizeMap = new HashMap<String,Point>();
					String leafFullContextName = getFullContextName(end, context, null);
					sizeMap.put(leafFullContextName, new Point(leafWidth, leafHeight));
				}
			}
			
			for(int j = 0 ; j < nContextDpeth ; j++) {
				for(i = 1 ; i <= flows.getCount() ; i++) {
					IRPFlow flow = (IRPFlow)flows.getItem(i);
					IRPTag contextTag = null;
					if(bSourceContext)
						contextTag = flow.getTag("SourceContext");
					else
						contextTag = flow.getTag("TargetContext");
					
					if(contextTag != null) {
						IRPCollection context = contextTag.getValueSpecifications();
						if(context != null && context.getCount() >= j+1) {
							IRPModelElement parent = null;
							IRPValueSpecification vs = (IRPValueSpecification) context.getItem(context.getCount() - j);
							if(vs instanceof IRPInstanceValue) {
								IRPInstanceValue iv = (IRPInstanceValue)vs;
								parent= iv.getValue();
							}
							String fullParentName = getFullContextName(parent,context, null);
							if(parent == null || sizeMap.get(fullParentName) != null)
								continue;
							
							List<IRPModelElement> childrenList = childrenMap.get(parent);
							if(childrenList != null && childrenList.size() > 0) {
								int xSize = 0;
								int ySize = 0;
								for(IRPModelElement child : childrenList) {
									String fullChildName = getFullContextName(child, context, parent);
									Point childSize = sizeMap.get(fullChildName);
									if(childSize != null) {
										xSize = childSize.x;
										ySize += childSize.y;
									}
								}
								if(xSize > 0 && ySize > 0) {
									xSize += xMargin * 2;
									ySize += yMargin * (childrenList.size() + 2);
									
									sizeMap.put(getFullContextName(parent, context, null), new Point(xSize, ySize));
								}
							}
						}
					}
				}
			}
		}
		return sizeMap;
	}

	String getFullContextName(IRPModelElement elem, IRPCollection context, IRPModelElement parent) {
		String fullContextName = null;
		if(elem != null && context != null) {
			String elemName = elem.getName();
			String contextStr = null;
			if(parent != null && Utils.getStereotype(parent, "ParticipantProperty") != null) {
				contextStr = parent.getName();
			}
			else if(context != null && context.getCount() > 0) {
				for(int j = 1 ; j <= context.getCount() ; j++) {
					IRPValueSpecification vs = (IRPValueSpecification) context.getItem(j);
					if(vs instanceof IRPInstanceValue) {
						IRPInstanceValue iv = (IRPInstanceValue)vs;
						IRPModelElement contextElem = iv.getValue();
						if(contextElem != null) {
							if(elem.equals(contextElem))
								break;
							if(contextStr == null)
								contextStr = contextElem.getName();
							else
								contextStr += "." + contextElem.getName();
						}
					}
				}
			}
			if(contextStr != null)
				fullContextName = contextStr + "." + elemName;
			else
				fullContextName = elemName;
		}
		return fullContextName;
	}
}
