package com.ibm.rhapsody.sysML.proxyportWizards;

import java.util.LinkedList;
import java.util.List;
import java.util.StringTokenizer;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;

import com.telelogic.rhapsody.core.HYPNameType;
import com.telelogic.rhapsody.core.IRPApplication;
import com.telelogic.rhapsody.core.IRPAssociationClass;
import com.telelogic.rhapsody.core.IRPAttribute;
import com.telelogic.rhapsody.core.IRPClassifier;
import com.telelogic.rhapsody.core.IRPCollection;
import com.telelogic.rhapsody.core.IRPDiagram;
import com.telelogic.rhapsody.core.IRPFlow;
import com.telelogic.rhapsody.core.IRPGraphEdge;
import com.telelogic.rhapsody.core.IRPGraphElement;
import com.telelogic.rhapsody.core.IRPGraphNode;
import com.telelogic.rhapsody.core.IRPGraphicalProperty;
import com.telelogic.rhapsody.core.IRPHyperLink;
import com.telelogic.rhapsody.core.IRPInstance;
import com.telelogic.rhapsody.core.IRPInstanceValue;
import com.telelogic.rhapsody.core.IRPModelElement;
import com.telelogic.rhapsody.core.IRPPackage;
import com.telelogic.rhapsody.core.IRPPort;
import com.telelogic.rhapsody.core.IRPProfile;
import com.telelogic.rhapsody.core.IRPProject;
import com.telelogic.rhapsody.core.IRPRelation;
import com.telelogic.rhapsody.core.IRPStereotype;
import com.telelogic.rhapsody.core.IRPTableLayout;
import com.telelogic.rhapsody.core.IRPTableView;
import com.telelogic.rhapsody.core.IRPTag;
import com.telelogic.rhapsody.core.IRPValueSpecification;
import com.telelogic.rhapsody.core.RhapsodyAppServer;

public class Utils {
	static IRPApplication rhapsody = null;
	
	public enum DirectionType {
		In,
		Out,
		InOut,
		Unspecified
	};
	
	public static IRPFlow findFlow(IRPClassifier block, String flowName) {
		IRPFlow flow = null;
		if(block != null) {
			IRPCollection flows = block.getFlows();
			if(flows != null && flows.getCount() > 0) {
				for(int i = 1 ; i <= flows.getCount() ; i++) {
					IRPFlow tmpFlow = (IRPFlow)flows.getItem(i);
					if(tmpFlow != null && tmpFlow.getName().equals(flowName)) {
						flow = tmpFlow;
						break;
					}
				}
			}
		}
		return flow;
	}

	public static IRPFlow findFlow(IRPClassifier block, IRPModelElement end1, IRPModelElement end2) {
		IRPFlow flow = null;
		if(block != null && end1 != null && end2 != null) {
			IRPCollection flows = block.getFlows();
			if(flows != null && flows.getCount() > 0) {
				for(int i = 1 ; i <= flows.getCount() ; i++) {
					IRPFlow tmpFlow = (IRPFlow)flows.getItem(i);
					IRPModelElement tmpEnd1 = tmpFlow.getEnd1();
					IRPModelElement tmpEnd1Port = tmpFlow.getEnd1Port();
					if(end1.equals(tmpEnd1) && end2.equals(tmpFlow.getEnd2())) {
						flow = tmpFlow;
						break;
					}
					else if(end1 instanceof IRPPort && tmpEnd1Port != null && end1.equals(tmpEnd1Port)) {
						IRPModelElement tmpEnd2 = tmpFlow.getEnd2();
						IRPModelElement tmpEnd2Port = tmpFlow.getEnd2Port();
						if(end2.equals(tmpEnd2) || end2.equals(tmpEnd2Port)) {
							flow = tmpFlow;
							break;
						}
					}
				}
			}
		}
		return flow;
	}
	
	public static IRPFlow findFlow(IRPPackage pkg, String flowName) {
		IRPFlow flow = null;
		if(pkg != null) {
			IRPCollection flows = pkg.getFlows();
			if(flows != null && flows.getCount() > 0) {
				for(int i = 1 ; i <= flows.getCount() ; i++) {
					IRPFlow tmpFlow = (IRPFlow)flows.getItem(i);
					if(tmpFlow != null && tmpFlow.getName().equals(flowName)) {
						flow = tmpFlow;
						break;
					}
				}
			}
		}
		return flow;
	}
	
	public static IRPFlow findFlowByEnd1NameAndContext(IRPClassifier block, String end1Name, List<IRPModelElement> end1Context, IRPModelElement end1) {
		IRPFlow flow = null;
		if(block != null && end1Name != null && end1Name.isEmpty() == false) {
			String name = end1Name;
			String type = null;
			StringTokenizer st = new StringTokenizer(end1Name, ":");
			if(st.hasMoreTokens())
				name = st.nextToken();
			if(st.hasMoreTokens())
				type = st.nextToken();
			
			IRPCollection flows = block.getFlows();
			if(flows != null && flows.getCount() > 0) {
				for(int i = 1 ; i <= flows.getCount() ; i++) {
					IRPFlow tmpFlow = (IRPFlow)flows.getItem(i);
					IRPModelElement tmpEnd1 = tmpFlow.getEnd1();
					if(tmpEnd1 != null && tmpEnd1.getName().equals(name) || (end1 != null && end1.equals(tmpEnd1))) {
						if(tmpEnd1 instanceof IRPAttribute) {
							IRPAttribute end1Attr = (IRPAttribute)tmpEnd1;
							IRPClassifier attrType = end1Attr.getType();
							if(attrType != null && attrType.getName().equals(type) == false) {
								continue;
							}
						}
						
						if(end1Context == null) {
							flow = tmpFlow;
							break;
						}
						else {
							String fullEnd1Name = getNameWithContext(tmpEnd1, end1Context);
							String flowName = tmpFlow.getName();
							if(flowName != null && fullEnd1Name != null && flowName.contains(fullEnd1Name)) {
								flow = tmpFlow;
								break;
							}	
						}
					}
				}
			}
		}
		return flow;
	}
	
	public static String getFullEnd2Name(IRPFlow flow, List<IRPModelElement> end1Context) {
		String fullEnd2Name = null;
		if(flow != null) {
			IRPModelElement end1 = flow.getEnd1();
			if(end1 != null) {
				String fullEnd1Name = getNameWithContext(end1, end1Context);
				String flowName = flow.getName();
				if(flowName != null && fullEnd1Name != null && flowName.contains(fullEnd1Name)) {
					fullEnd2Name = flowName.replaceFirst(fullEnd1Name + "_", "");
					if(fullEnd2Name != null && end1 instanceof IRPAttribute) {
						IRPAttribute attr = (IRPAttribute)end1;
						IRPClassifier type = attr.getType();
						if(type != null)
							fullEnd2Name += ":" + type.getName(); 
					}
				}
			}
		}
		return fullEnd2Name;
	}
	
	public static IRPPackage getPackageOfBlock(IRPClassifier block) {
		IRPPackage pkg = null;
		if(block != null) {
			IRPModelElement owner = block.getOwner();
			while(owner != null) {
				if(owner instanceof IRPPackage) {
					pkg = (IRPPackage)owner;
					break;
				}
				owner = owner.getOwner();
			}
		}
		return pkg;
	}
	
	public static boolean isProxyport(IRPPort port) {
		boolean bIsProxyport = false;
		if(port != null) {
			IRPCollection stereotypes = port.getStereotypes();
			if(stereotypes != null && stereotypes.getCount() > 0) {
				for(int i = 1 ; i <= stereotypes.getCount() ; i++) {
					IRPStereotype s = (IRPStereotype) stereotypes.getItem(i);
					if(s != null && s.getName().equals("proxyPort")) {
						bIsProxyport = true;
						break;
					}
				}
			}
		}
		return bIsProxyport;
	}
	
	public static boolean isInterfaceBlock(IRPClassifier block) {
		boolean bIsInterfaceBlock = false;
		if(block != null) {
			IRPCollection stereotypes = block.getStereotypes();
			if(stereotypes != null && stereotypes.getCount() > 0) {
				for(int i = 1 ; i <= stereotypes.getCount() ; i++) {
					IRPStereotype s = (IRPStereotype) stereotypes.getItem(i);
					if(s != null && s.getName().equals("interfaceBlock")) {
						bIsInterfaceBlock = true;
						break;
					}
				}
			}
		}
		return bIsInterfaceBlock;
	}
	
	public static IRPStereotype getStereotype(IRPModelElement elem, String stereotypeName) {
		IRPStereotype stereotype = null;
		if(elem != null && stereotypeName != null && stereotypeName.isEmpty() == false) {
			IRPCollection stereotypes = elem.getStereotypes();
			if(stereotypes != null && stereotypes.getCount() > 0) {
				for(int i = 1 ; i <= stereotypes.getCount() ; i++) {
					IRPStereotype s = (IRPStereotype) stereotypes.getItem(i);
					if(s != null && s.getName().equals(stereotypeName)) {
						stereotype = s;
						break;
					}
				}
			}
		}
		return stereotype;
	}
	
	public static IRPModelElement getInterfaceBlockProperty(IRPModelElement property, IRPModelElement end1Part, IRPClassifier block, IRPClassifier interfaceBlock) {
		IRPModelElement interfaceBlockProp = null;
		if(block != null && interfaceBlock != null & property != null) {
			IRPCollection flows = block.getFlows();
			IRPPackage pkg = Utils.getPackageOfBlock(block);
			if(pkg != null) {
				IRPCollection pkgFlows = pkg.getFlows();
				if(pkgFlows != null && pkgFlows.getCount() > 0) {
					for(int i = 1 ; i <= pkgFlows.getCount() ; i++) {
						flows.addItem((IRPModelElement)pkgFlows.getItem(i));
					}
				}
			}
			if(flows != null && flows.getCount() > 0) {
				for(int i = 1 ; i <= flows.getCount() ; i++) {
					IRPFlow flow = (IRPFlow)flows.getItem(i);
					IRPModelElement end2 = null;
					IRPModelElement end1 = flow.getEnd1();
					IRPPort end1Port = flow.getEnd1Port();
					if(end1 != null && end1.equals(property))
						 end2 = flow.getEnd2();
					else if(property instanceof IRPPort &&
							end1Port != null && end1Port.equals(property) &&
							end1Part != null && end1Part.equals(end1)) {
						if(flow.getEnd2() instanceof IRPPort)
							end2 = flow.getEnd2();
						else
							end2 = flow.getEnd2Port();
					}
					
					if(end2 != null && end2.getOwner().equals(interfaceBlock)) {
						interfaceBlockProp = end2;
						break;	
					}
				}
			}
		}
		return interfaceBlockProp;
	}
	
	public static IRPAttribute findAttributeByNameAndType(IRPClassifier block, String attrNameAndType) {
		IRPAttribute attr = null;
		if(block != null && attrNameAndType != null && attrNameAndType.isEmpty() == false) {
			StringTokenizer st = new StringTokenizer(attrNameAndType, ":");
			String attrName = null;
			String attrType = null;
			if(st != null) {
				if(st.hasMoreTokens())
					attrName = st.nextToken();
				if(st.hasMoreTokens())
					attrType = st.nextToken();
			}
			
			if(attrName != null) {
				IRPAttribute tmpAttr = block.findAttribute(attrName);
				if(tmpAttr != null) {
					if(attrType != null) {
						IRPClassifier type = tmpAttr.getType();
						if(type != null && type.getName().equals(attrType))
							attr = tmpAttr;
					}
					else
						attr = tmpAttr;
				}
			}
		}
		return attr;
	}
	
	public static String getNameAndTypeString(IRPAttribute attr) {
		String nameAndType = null;
		if(attr != null) {
			nameAndType = attr.getName();
			IRPClassifier type = attr.getType();
			if(type != null)
				nameAndType += ":" + type.getName();
		}
		return nameAndType;
	}
	
	public static IRPDiagram getCurrentDiagram() {
		IRPDiagram diagram = null;
		getRhapsody();
		if(rhapsody != null) {
			IRPModelElement selectedElem = rhapsody.getSelectedElement();
			IRPGraphElement graphElem = getSelectedGraphicalElement(selectedElem);
			diagram = rhapsody.getDiagramOfSelectedElement();
			if(diagram == null && graphElem != null)
				diagram = graphElem.getDiagram();
		}
		return diagram;
	}
	
	public static IRPGraphElement getGraphicalElement(IRPDiagram diagram, IRPModelElement modelElement, IRPModelElement graphicalParent) {
		IRPGraphElement graphElement = null;
		if(diagram != null && modelElement != null) {
			IRPCollection graphicalElements = diagram.getGraphicalElements();
			if(graphicalElements != null) {
				for(int i = 1 ; i <= graphicalElements.getCount() ; i++) {
					IRPGraphElement elem = (IRPGraphElement) graphicalElements.getItem(i);
					if(elem.getModelObject() != null && elem.getModelObject().equals(modelElement)) {
						IRPGraphElement parent = elem.getGraphicalParent();
						if(graphicalParent == null || parent == null) {
							graphElement = elem;
							break;
						}
						else {
							 IRPModelElement parentMdlElem = parent.getModelObject();
							 if(parentMdlElem != null && parentMdlElem.equals(graphicalParent)) {
								 graphElement = elem;
								 break;
							 }
						}
					}
				}
			}
		}
		return graphElement;
	}
	
	public static IRPGraphNode getGraphicalNode(IRPDiagram diagram, IRPModelElement modelElement, IRPGraphNode graphicalParent, IRPModelElement modelParentElement) {
		IRPGraphNode graphElement = null;
		if(diagram != null && modelElement != null) {
			IRPCollection graphicalElements = diagram.getGraphicalElements();
			if(graphicalElements != null) {
				for(int i = 1 ; i <= graphicalElements.getCount() ; i++) {
					IRPGraphElement elem = (IRPGraphElement)graphicalElements.getItem(i);
					if(elem instanceof IRPGraphNode) {
						IRPGraphNode node = (IRPGraphNode) elem;
						IRPModelElement mdlElem = node.getModelObject();
						if(mdlElem != null && mdlElem.equals(modelElement)) {
							IRPGraphElement parent = node.getGraphicalParent();
							if(graphicalParent == null && modelParentElement == null) {
								graphElement = node;
								break;
							}
							else if(parent != null && parent instanceof IRPGraphNode) {
								IRPGraphNode nodeParent = (IRPGraphNode)parent;
								IRPModelElement parentMdlElem = nodeParent.getModelObject();
								if(graphicalParent != null && graphicalParent.equals(nodeParent)) {
									graphElement = node;
									break;
								}
								else if(modelParentElement != null && modelParentElement.equals(parentMdlElem)) {
									graphElement = node;
									break;
								}
							}
						}
					}
				}
			}
		}
		return graphElement;
	}
	
	public static IRPGraphNode getGraphicalNode(IRPDiagram diagram, IRPModelElement modelElement, IRPCollection context) {
		IRPGraphNode graphNode = null;
		if(diagram != null && modelElement != null) {
			if(context == null || context.getCount() == 0)
				return getGraphicalNode(diagram, modelElement, null, null);
			
			IRPCollection graphicalElements = diagram.getGraphicalElements();
			if(graphicalElements != null) {
				for(int i = 1 ; i <= graphicalElements.getCount() ; i++) {
					IRPGraphElement graphElem = (IRPGraphElement) graphicalElements.getItem(i);
					if(graphElem instanceof IRPGraphNode && graphElem.getModelObject() != null && graphElem.getModelObject().equals(modelElement)) {
						IRPGraphNode tmpGraphNode = (IRPGraphNode)graphElem;
												
						boolean bContextOk = true;
						for(int j = 0 ; j < context.getCount() ; j++) {
							IRPValueSpecification vs = (IRPValueSpecification) context.getItem(context.getCount() - j);
							if(vs instanceof IRPInstanceValue) {
								IRPInstanceValue iv = (IRPInstanceValue)vs;
								IRPModelElement contextElem = iv.getValue();
								if(contextElem != null) {
									IRPGraphNode graphicalParent = (IRPGraphNode) tmpGraphNode.getGraphicalParent();
									if(graphicalParent != null && graphicalParent.getModelObject() != null && graphicalParent.getModelObject().equals(contextElem)) {
										tmpGraphNode = graphicalParent;
									}
									else {
										bContextOk = false;
										break;
									}
								}
							}
						}
						
						if(bContextOk) {
							graphNode = (IRPGraphNode) graphElem;
							break;
						}
					}
				}
			}
		}
		return graphNode;
	}
	
	public static IRPGraphElement getSelectedGraphicalElement(IRPModelElement modelElement) {
		IRPGraphElement selectedElem = null;
		getRhapsody();
		if(rhapsody != null) {
			IRPCollection elems = rhapsody.getSelectedGraphElements();
			if(elems != null && elems.getCount() == 1) {
				Object elem = elems.getItem(1);
				if(elem instanceof IRPGraphElement) {
					IRPGraphElement node = (IRPGraphElement)elem;
					if(node.getModelObject() != null && node.getModelObject().equals(modelElement)) {
						selectedElem = node;
					}
				}
			}
		}
		return selectedElem;
	}
	
	public static Point getSourceTargetPoint(IRPGraphEdge edge, boolean bGetTarget) {
		Point pt = null;
		if(edge != null) {
			IRPGraphicalProperty prp = null;
			if(bGetTarget)
				prp = edge.getGraphicalProperty("TargetPosition");
			else
				prp = edge.getGraphicalProperty("SourcePosition");

			if(prp != null) {
				pt = extractPoint(prp.getValue());
			}
		}
		return pt;
	}
	
	public static Point extractPoint(String val) {
		Point pt = null;
		if(val != null) {
			StringTokenizer st = new StringTokenizer(val, ",");
			String xVal = st.nextToken();
			String yVal = st.nextToken();

			int x = Integer.parseInt(xVal);
			int y = Integer.parseInt(yVal);

			pt = new Point(x,y);
		}
		return pt;
	}
	
	public static void moveNodeToPoint(IRPGraphNode node, Point point) {
		if(node != null && point != null) {
			Integer x = point.x;
			Integer y = point.y;
			String ptStr = x.toString() + "," + y.toString();
			node.setGraphicalProperty("Position", ptStr);
		}
	}
	
	public static Point getMiddlePointOfGraphicalNode(IRPGraphNode node, boolean bForPort) {
		Point ptMiddle = null;
		if(node != null) {
			IRPGraphicalProperty prp = node.getGraphicalProperty("Position");
			if(prp != null) {
				if(bForPort)
					ptMiddle = extractPoint(prp.getValue());
				else {
					Point ptTopLeft = extractPoint(prp.getValue());
					int width = 0;
					int height = 0;
					prp = node.getGraphicalProperty("Width");
					if(prp != null && prp.getValue() != null) {
						width = Integer.parseInt(prp.getValue());
					}
					prp = node.getGraphicalProperty("Height");
					if(prp != null && prp.getValue() != null) {
						height = Integer.parseInt(prp.getValue());
					}
					
					if(ptTopLeft != null && width > 0 && height > 0) {
						int xMiddle = ptTopLeft.x + width/2;
						int yMiddle = ptTopLeft.y + height/2;
						ptMiddle = new Point(xMiddle, yMiddle);
					}
				}
			}
		}
		return ptMiddle;
	}
	
	public static Point getCandidatePointForPort(IRPGraphNode node) {
		Point pt = null;
		if(node != null) {
			Point ptTopLeft = null;
			int width = 0;
			int height = 0;
			IRPGraphicalProperty prp = node.getGraphicalProperty("Position");
			if(prp != null) {
				ptTopLeft = extractPoint(prp.getValue());
			}
			prp = node.getGraphicalProperty("Width");
			if(prp != null && prp.getValue() != null) {
				width = Integer.parseInt(prp.getValue());
			}
			prp = node.getGraphicalProperty("Height");
			if(prp != null && prp.getValue() != null) {
				height = Integer.parseInt(prp.getValue());
			}
			
			if(ptTopLeft != null && width > 0 && height > 0) {
				int xPos = ptTopLeft.x + width;
				int yPos = ptTopLeft.y + height/2;
				pt = new Point(xPos, yPos);
			}
		}
		return pt;
	}
	
	public static IRPCollection getAllPartsWithProxyPorts(IRPClassifier block) {
		IRPCollection parts = null;
		IRPCollection relations = block.getRelations();
		if(relations != null && relations.getCount() > 0) {
			for(int i = 1 ; i <= relations.getCount() ; i++) {
				IRPRelation rel = (IRPRelation)relations.getItem(i);
				if(rel instanceof IRPInstance) {
					IRPInstance part = (IRPInstance)rel;
					List<IRPPort> ports = getProxyportsOfPartType(part);
					if(ports != null && ports.size() > 0) {
						if(parts == null)
							parts = createNewIRPCollection();
						parts.addItem(part);
					}
				}
			}
		}
		return parts;
	}

	public static List<IRPPort> getProxyportsOfPartType(IRPInstance part) {
		List<IRPPort> proxyports = null;
		if(part != null) {
			IRPClassifier type = part.getOtherClass();
			if(type != null) {
				proxyports = getProxyportsOfBlock(type);
			}
		}
		return proxyports;
	}

	public static List<IRPPort> getProxyportsOfBlock(IRPClassifier block) {
		List<IRPPort> proxyports = null;
		if(block != null) {
			IRPCollection ports = block.getPorts();
			if(ports != null && ports.getCount() > 0) {
				for(int j = 1 ; j <= ports.getCount() ; j++) {
					IRPPort port = (IRPPort) ports.getItem(j);
					IRPCollection stereotypes = port.getStereotypes();
					if(stereotypes != null && stereotypes.getCount() > 0) {
						for(int i = 1 ; i <= stereotypes.getCount() ; i++) {
							IRPStereotype s = (IRPStereotype) stereotypes.getItem(i);
							if(s != null && s.getName().equals("proxyPort")) {
								if(proxyports == null)
									proxyports = new LinkedList<IRPPort>();
								proxyports.add(port);
								break;
							}
						}
					}
				}
			}
		}
		return proxyports;
	}
	
	public static IRPPort findProxyportByName(IRPClassifier block, String portName) {
		IRPPort proxyport = null;
		if(block != null && portName != null && portName.isEmpty() == false) {
			List<IRPPort> ports = Utils.getProxyportsOfBlock(block);
			if(ports != null && ports.size() > 0) {
				for(IRPPort port : ports) {
					if(port.getName().equals(portName)) {
						proxyport = port;
						break;
					}
				}
			}
		}
		return proxyport;
	}
	
	public static String createFlowNameWithContext(IRPModelElement end1, List<IRPModelElement> end1Context,
			IRPModelElement end2, List<IRPModelElement> end2Context) {
		String flowName = null;
		if(end1 != null && end2 != null) {
			String end1FullName = getNameWithContext(end1, end1Context);
						
			String end2FullName = getNameWithContext(end2, end2Context);
						
			flowName = end1FullName + "_" + end2FullName;
		}
		return flowName;
	}
	
	public static String getNameWithContext(IRPModelElement elem, List<IRPModelElement> context) {
		String nameWithContext = null;
		if(elem != null) {
			if(context != null) {
				for(int i = 0 ; i < context.size() ; i++) {
					IRPModelElement contextElem = context.get(i);
					if(nameWithContext == null)
						nameWithContext = contextElem.getName();
					else 
						nameWithContext += "." + contextElem.getName();
				}
			}
			if(nameWithContext != null && nameWithContext.isEmpty() == false)
				nameWithContext += "." + elem.getName();
			else
				nameWithContext = elem.getName();
		}
		return nameWithContext;
	}
	
	public static void addContextToTag(IRPTag tag, List<IRPModelElement> context) {
		if(tag != null && context != null) {
			for(int i = 0 ; i < context.size() ; i++) {
				IRPModelElement elem = context.get(i);
				tag.addElementDefaultValue(elem);
			}
		}
	}
	
	public static IRPInstance getParticipantPropertyOf(IRPAssociationClass assocClass, IRPRelation relation) {
		IRPInstance participantProperty = null;
		if(assocClass != null && relation != null) {
			IRPCollection relations = assocClass.getRelations();
			if(relations != null && relations.getCount() > 0) {
				for(int i = 1 ; i <= relations.getCount() ; i++) {
					IRPRelation rel = (IRPRelation) relations.getItem(i);
					if(rel instanceof IRPInstance) {
						IRPInstance part = (IRPInstance)rel;
						if(getStereotype(part, "ParticipantProperty") != null && part.getName().equals(relation.getName())) {
							participantProperty = part;
							break;
						}
					}
				}
			}
		}
		return participantProperty;
	}
	
	public static IRPTableLayout getInputOutputTableLayout () {
		IRPTableLayout inputOutputTableLayout = null;
		getRhapsody();
		if(rhapsody != null) {
			IRPProject prj = rhapsody.activeProject();
			if(prj != null) {
				IRPCollection profiles = prj.getProfiles();
				if(profiles != null && profiles.getCount() > 0) {
					for(int i = 1 ; i <= profiles.getCount() ; i++) {
						IRPProfile profile = (IRPProfile) profiles.getItem(i);
						if(profile.getName().equals("SysML")) {
							IRPModelElement elem = profile.findNestedElementRecursive("InputOutputLayout", "TableLayout");
							if(elem != null && elem instanceof IRPTableLayout) {
								inputOutputTableLayout = (IRPTableLayout)elem;
								break;
							}
						}
					}
				}
			}
		}
		return inputOutputTableLayout;
	}
	
	public static IRPTableView getOrCreateTableView(IRPModelElement owner, String name, IRPTableLayout layout, IRPModelElement scope) {
		IRPTableView tableView = null;
		if(owner != null && name != null && layout != null) {
			IRPModelElement elem = owner.findNestedElement(name, "InputOutputTable");
			if(elem != null && elem instanceof IRPTableView) {
				IRPTableView tmpTableView = (IRPTableView)elem;
				if(tmpTableView.getItsTableLayout().equals(layout)) {
					tableView = tmpTableView;
				}
			}
			
			if(tableView == null) {
				tableView = (IRPTableView) owner.addNewAggr("InputOutputTable", name);
				tableView.setItsTableLayout(layout);
				
				if(scope != null) {
					IRPCollection col = createNewIRPCollection();
					col.addItem(scope);
					tableView.setScope(col);
				}
			}
		}
		return tableView;
	}
	
	public static IRPHyperLink getOrCreateHyperLink(IRPModelElement owner, IRPModelElement target) {
		IRPHyperLink hyperLink = null;
		if(owner != null && target != null) {
			String name = target.getName();
			IRPModelElement elem = owner.findNestedElement(name, "HyperLink");
			if(elem != null && elem instanceof IRPHyperLink) {
				IRPHyperLink hl = (IRPHyperLink)elem;
				if(hl.getTarget() != null && hl.getTarget().equals(target)) {
					hyperLink = hl;
				}
			}
			
			if(hyperLink == null) {
				hyperLink = (IRPHyperLink) owner.addNewAggr("HyperLink", name);
				hyperLink.setTarget(target);
				hyperLink.setDisplayOption(HYPNameType.RP_HYP_NAMETEXT,name);
			}
		}
		return hyperLink;
	}
	
	public static String getUniqueName(IRPModelElement owner, String baseName, String metaClass) {
		String name = null;
		if(owner != null && baseName != null && metaClass != null) {
			name = baseName;
			IRPModelElement existingElem = owner.findNestedElement(name, metaClass);
			int i = 0;
			while(existingElem != null && i < 20) {
				i++;
				name = baseName + "_" + Integer.toString(i);
				existingElem = owner.findNestedElement(name, metaClass);
			}
		}
		return name;
	}
	
	public static String getDirectionString(IRPModelElement elem) {
		String direction = null;
		if(elem != null) {
			IRPTag directionTag = elem.getTag("direction");
			if(directionTag != null) {
				direction = directionTag.getValue();
				if(direction != null && direction.equals("required"))
					direction = "reqd";
				else if(direction != null && direction.equals("provided"))
					direction = "prov";
				else if(direction != null && direction.equals("providedrequired"))
					direction = "provreqd";
			}
		}
		return direction;
	}
	
	public static DirectionType getDirectionType(IRPModelElement elem) {
		DirectionType direction = DirectionType.Unspecified;
		if(elem != null) {
			IRPTag directionTag = elem.getTag("direction");
			if(directionTag != null) {
				String directionStr = directionTag.getValue();		
				if(directionStr != null && directionStr.equals("In"))
					direction = DirectionType.In;
				else if(directionStr != null && directionStr.equals("Out"))
					direction = DirectionType.Out;
			}
		}
		return direction;
	}
	
	public static List<IRPClassifier> getAvailableInterfaceBlocks(IRPClassifier block) {
		List<IRPClassifier> interfaceBlocks = null;
		IRPPackage pkg = getPackageOfBlock(block);
		if(pkg != null) {
			IRPCollection classes = pkg.getClasses();
			if(classes != null && classes.getCount() > 0) {
				for(int i = 1 ; i <= classes.getCount() ; i++) {
					IRPClassifier cls = (IRPClassifier)classes.getItem(i);
					if(cls != null && isInterfaceBlock(cls)) {
						if(interfaceBlocks == null)
							interfaceBlocks = new LinkedList<IRPClassifier>();
						interfaceBlocks.add(cls);
					}
				}
			}
		}
		return interfaceBlocks;
	}
	
	public static IRPCollection createNewIRPCollection() {
		IRPCollection collection = null;
		getRhapsody();
		if(rhapsody != null)
			collection = rhapsody.createNewCollection();
		return collection;
	}
	
	public static IRPApplication getRhapsody() {
		if(rhapsody == null)
			rhapsody = RhapsodyAppServer.getActiveRhapsodyApplication();
		return rhapsody;
	}
	
	public static void setRhapsody(IRPApplication rhapApp) {
		rhapsody = rhapApp;
	}
	
	public static TreeItem findTreeItemByName(TreeItem[] items, String name) {
		TreeItem item = null;
		if(items != null && items.length > 0 && name != null && name.isEmpty() == false) {
			for(TreeItem innerItem : items) {
				if(innerItem.getText() != null && innerItem.getText().equals(name)) {
					item = innerItem;
					break;
				}
			}
		}
		return item;
	}
	
	public static TreeItem getOrCreateTreeItem(TreeItem root, String name, List<TreeItem> treeItems) {
		TreeItem item = null;
		if(root != null && name != null && name.isEmpty() == false) {
			item = findTreeItemByName(root.getItems(), name);
			if(item == null) {
				item = new TreeItem(root, SWT.NONE);
				item.setText(0, name);
				if(treeItems != null)
					treeItems.add(item);
			}
		}
		return item;
	}
	
	public static TreeItem getOrCreateTreeRootItem(Tree tree, String name) {
		TreeItem item = null;
		if(tree != null && name != null && name.isEmpty() == false) {
			item = findTreeItemByName(tree.getItems(), name);
			if(item == null) {
				item = new TreeItem(tree, SWT.NONE);
				item.setText(0, name);
			}
		}
		return item;
	}
	
	public static IRPInstance getTreePartContext(TreeItem treeItem) {
		IRPInstance partContext = null;
		if(treeItem != null) {
			TreeItem parent = treeItem.getParentItem();
			if(parent != null && parent.getText() != null && parent.getText().equals("Proxyports")) {
				TreeItem partItem = parent.getParentItem();
				if(partItem != null && partItem.getData() != null && partItem.getData() instanceof IRPInstance) {
					partContext = (IRPInstance)partItem.getData();
				}
			}
		}
		return partContext;
	}
}
