
//Licensed Materials - Property of IBM 
// Copyright IBM Corporation 2006, 2008. All Rights Reserved.

package com.telelogic.rhapsody.core;



public interface IRPTableLayout  extends IRPUnit {
	/**
 	 * Adds a new column to the table layout.
 	 * @param type the column Type (equivalent to Type field in the UI) - the valid values for this parameter are the constants that are defined in the class IRPTableLayout.Column, for example, IRPTableLayout.Column.ANNOTATION_ATTRIBUTE.
 	 * @param Property the column Property (equivalent to Property field in the UI) - the valid values for this parameter are the constants defined in the classes nested beneath the class IRPTableLayout.Column. The nested class to use depends upon the value used for the "type" parameter.
 	 * <p>
 	 * For example, if the value of the "type" parameter is IRPTableLayout.Column.ANNOTATION_ATTRIBUTE, the valid values for the "Property" parameter are the constants defined in the nested class IRPTableLayout.Column.AnnotationAttribute, such as IRPTableLayout.Column.AnnotationAttribute.ID and IRPTableLayout.Column.AnnotationAttribute.SPECIFICATION.
 	 * <p>
 	 * However, if you specified TAG, TAG_EDIT, TAG_EDIT_STRICT, or USER_DEFINED_METHOD as the "type" parameter, you must provide a string that reflects the full path of the relevant tag or the name of the relevant plugin method.
 	 * <p>
 	 * For TAG, TAG_EDIT, and TAG_EDIT_STRICT, use the string that is returned when you call the method getFullPathName for the tag.
 	 * <p>
 	 * If you  specified USER_DEFINED_METHOD as the type, the Property parameter should take the form pluginName.method. (If you do not specify the plugin name, all plugins will be searched until a method with the specified name is found).
 	 * @param ColumnName the text to use as the heading for the column
 	 */
	public void addColumn(String type, String Property, String ColumnName);
	/**
 	 * Checks whether the first column of the layout includes controls for collapsing and expanding rows that have the same value in the first column.
 	 * @return 1 if the first column includes collapse/expand controls, 0 otherwise
 	 */
	public int getCollapseFirstColumn();
	/**
	 * method GetColumns
	 * @throws RhapsodyRuntimeException
	 */
	public IRPCollection getColumns();
	/**
 	 * Returns a collection of the element types that were specified to be displayed in the table. The collection consists of strings (from the list of types displayed on the ElementTypes tab of the Features window for table layouts).
 	 * @return the element types that were specified to be displayed in the table
 	 */
	public IRPCollection getElementTypes();
	/**
 	 * For "relation tables", returns a collection of the element types specified as the "from" element types. The collection consists of strings (from the list of types displayed on the From Element Types tab of the Features window for table layouts).
 	 * @return the types specified as the "from" element types for the table layout
 	 */
	public IRPCollection getFromElementTypes();
	/**
 	 * For "relation tables", returns the query that was specified to determine the "from" element types.
 	 * @return the query that was specified to determine the "from" element types for the table layout
 	 */
	public IRPTableLayout getFromElementTypesQueryToUse();
	/**
 	 * For "relation tables", checks whether a query or collection of element types was used to specify the "from" element types.
 	 * @return one of the constants contained in the class IRPTableLayout.QueryOrElementsList: QUERY if a query was used, ELEMENTS_LIST if a collection of element types was used.
 	 */
	public int getFromElementTypesUseQueryOrElementsList();
	/**
 	 * Checks whether the table was defined as a "relation table".
 	 * @return 1 if the table was defined as a "relation table", 0 otherwise
 	 */
	public int getRelationTable();
	/**
	 * method GetResultList
	 * @throws RhapsodyRuntimeException
	 */
	public IRPCollection getResultList(IRPModelElement scope);
	/**
 	 * For "relation tables", returns a collection of the element types specified as the "to" element types. The collection consists of strings (from the list of types displayed on the To Element Types tab of the Features window for table layouts).
 	 * @return the types specified as the "to" element types for the table layout
 	 */
	public IRPCollection getToElementTypes();
	/**
 	 * For "relation tables", returns the query that was specified to determine the "to" element types.
 	 * @return the query that was specified to determine the "to" element types for the table layout
 	 */
	public IRPTableLayout getToElementTypesQueryToUse();
	/**
 	 * For "relation tables", checks whether a query or collection of element types was used to specify the "to" element types.
 	 * @return one of the constants contained in the class IRPTableLayout.QueryOrElementsList: QUERY if a query was used, ELEMENTS_LIST if a collection of element types was used.
 	 */
	public int getToElementTypesUseQueryOrElementsList();
	/**
 	 * Removes the specified column from the table layout.
 	 * @param Index the index representing the position of the column in the table. The index for the fist column in the table is 0.
 	 */
	public void removeColumn(int Index);
	/**
 	 * Specifies whether or not the fist column should include controls for collapsing and expanding rows that have the same value in the first column.
 	 * @param collapse use 1 if the first column should include collapse/expand controls, 0 otherwise.
 	 */
	public void setCollapseFirstColumn(int collapse);
	/**
 	 * Specifies the list of element types that should be displayed in the table. The parameter must be a collection of strings (from the list of types displayed on the ElementTypes tab of the Features window for table layouts).
 	 * @param elements the element types that should be displayed in the table
 	 */
	public void setElementTypes(IRPCollection elements);
	/**
 	 * For "relation tables", specifies the list of element types to use as the "from" element types. The parameter must be a collection of strings (from the list of types displayed on the From Element Types tab of the Features window for table layouts).
 	 * @param elements collection of element types to use as the "from" element types for the table layout
 	 */
	public void setFromElementTypes(IRPCollection elements);
	/**
 	 * For "relation tables", specifies the query to use to determine the "from" element types for the table layout.
 	 * @param query the query to use to determine the "from" element types for the table layout. To clear a previous query, use null for the parameter.
 	 */
	public void setFromElementTypesQueryToUse(IRPTableLayout query);
	/**
 	 * For "relation tables", specifies whether a query or collection of element types should be used to determine the "from" element types for the table layout.
 	 * @param queryOrElementsList one of the constants contained in the class IRPTableLayout.QueryOrElementsList: QUERY if a query should be used, ELEMENTS_LIST if a collection of element types should be used.
 	 */
	public void setFromElementTypesUseQueryOrElementsList(int queryOrElementsList);
	/**
 	 * Specifies whether the table should be defined as a "relation table".
 	 * @param relation use 1 if the table should be defined as a "relation table", 0 otherwise.
 	 */
	public void setRelationTable(int relation);
	/**
 	 * For "relation tables", specifies the list of element types to use as the "to" element types for the table layout. The parameter must be a collection of strings (from the list of types displayed on the To Element Types tab of the Features window for table layouts).
 	 * @param elements collection of element types to use as the "to" element types for the table layout
 	 */
	public void setToElementTypes(IRPCollection elements);
	/**
 	 * For "relation tables", specifies the query to use to determine the "to" element types for the table layout.
 	 * @param query the query to use to determine the "to" element types for the table layout. To clear a previous query, use null for the parameter.
 	 */
	public void setToElementTypesQueryToUse(IRPTableLayout query);
	/**
 	 * For "relation tables", specifies whether a query or collection of element types should be used to determine the "to" element types for the table layout.
 	 * @param queryOrElementsList one of the constants contained in the class IRPTableLayout.QueryOrElementsList: QUERY if a query should be used, ELEMENTS_LIST if a collection of element types should be used.
 	 */
	public void setToElementTypesUseQueryOrElementsList(int queryOrElementsList);
	/**
	 * get property reserve1
	 * @throws RhapsodyRuntimeException
	 */
	public String getReserve1();
	
	/** Constant values used with elements of this type **/
	/**
This class holds constant values to be used with addColumn method.
	**/
	public static final class Column
	{
		/**
Value used for Type parameter of addColumn method

		**/
		public static final	 String		GENERAL_ATTRIBUTE		= "General Attribute";
		/**
Value used for Type parameter of addColumn method

		**/
		public static final	 String		RELAION_ATTRIBUTE_FROM		= "Relation Attribute (From)";
		/**
Value used for Type parameter of addColumn method

		**/
		public static final	 String		RELAION_ATTRIBUTE_TO		= "Relation Attribute (To)";
		/**
Value used for Type parameter of addColumn method

		**/
		public static final	 String		REQUIREMENT_ATTRIBUTE		= "Requirement Attribute";
		/**
Value used for Type parameter of addColumn method
		**/
		public static final	 String		ANNOTATION_ATTRIBUTE		= "Annotation Attribute";
		/**
Value used for Type parameter of addColumn method

		**/
		public static final	 String		FLOW_ATTRIBUTE		= "Flow Attribute";
		/**
Value used for Type parameter of addColumn method

		**/
		public static final	 String		TAG		= "Tag";
		/**
Value used for Type parameter of addColumn method

		**/
		public static final	 String		TAG_EDIT		= "Tag (Edit)";
		/**
Value used for Type parameter of addColumn method

		**/
		public static final	 String		TAG_EDIT_STRICT		= "Tag (Edit, Strict)";
		/**
Value used for Type parameter of addColumn method

		**/
		public static final	 String		DEPENDS_ON		= "Depends On";
		/**
Value used for Type parameter of addColumn method

		**/
		public static final	 String		USER_DEFINED_METHOD		= "User Defined Method";
		/**
Contains values to be used for Property parameter of addColumn method, 
when AnnotationAttribute is selected for the Type parameter of addColumn method.
		**/
		public static final class AnnotationAttribute
		{
			/**
Value to be used for Property parameter of addColumn method, 
when AnnotationAttribute is selected for the Type parameter of addColumn method.

			**/
			public static final	 String		ID		= "ID";
			/**
Value to be used for Property parameter of addColumn method, 
when AnnotationAttribute is selected for the Type parameter of addColumn method.

			**/
			public static final	 String		SPECIFICATION		= "Specification";
		}
		/**
Contains values to be used for Property parameter of addColumn method, 
when FlowAttribute is selected for the Type parameter of addColumn method.

		**/
		public static final class FlowAttribute
		{
			/**
Value to be used for Property parameter of addColumn method, 
when FlowAttribute is selected for the Type parameter of addColumn method.

			**/
			public static final	 String		ITEM_FLOWS		= "Item Flows";
		}
		/**
Contains values to be used for Property parameter of addColumn method, 
when GeneralAttribute is selected for the Type parameter of addColumn method.

		**/
		public static final class GeneralAttribute
		{
			/**
Value to be used for Property parameter of addColumn method, 
when GeneralAttribute is selected for the Type parameter of addColumn method.
			**/
			public static final	 String		NAME		= "Name";
			/**
Value to be used for Property parameter of addColumn method, 
when GeneralAttribute is selected for the Type parameter of addColumn method.
			**/
			public static final	 String		OWNER		= "Owner";
			/**
Value to be used for Property parameter of addColumn method, 
when GeneralAttribute is selected for the Type parameter of addColumn method.
			**/
			public static final	 String		DESCRIPTION		= "Description";
			/**
Value to be used for Property parameter of addColumn method, 
when GeneralAttribute is selected for the Type parameter of addColumn method.
			**/
			public static final	 String		ELEMENT_TYPE		= "Element type";
			/**
Value to be used for Property parameter of addColumn method, 
when GeneralAttribute is selected for the Type parameter of addColumn method.
			**/
			public static final	 String		STEREOTYPES		= "Stereotypes";
			/**
Value to be used for Property parameter of addColumn method, 
when GeneralAttribute is selected for the Type parameter of addColumn method.

			**/
			public static final	 String		CLASSIFIER		= "Classifier";
		}
		/**
Contains values to be used for Property parameter of addColumn method, 
when RelationAttributeFrom is selected for the Type parameter of addColumn method.

		**/
		public static final class RelationAttributeFrom
		{
			/**
Value to be used for Property parameter of addColumn method, 
when RelationAttributeFrom is selected for the Type parameter of addColumn method.
			**/
			public static final	 String		VIA_PORT		= "Via port";
			/**
Value to be used for Property parameter of addColumn method, 
when RelationAttributeFrom is selected for the Type parameter of addColumn method.
			**/
			public static final	 String		PORT_PROVIDED_INERFACE		= "Port provided interface";
			/**
Value to be used for Property parameter of addColumn method, 
when RelationAttributeFrom is selected for the Type parameter of addColumn method.
			**/
			public static final	 String		FROM_ELEMENT		= "From element";
			/**
Value to be used for Property parameter of addColumn method, 
when RelationAttributeFrom is selected for the Type parameter of addColumn method.

			**/
			public static final	 String		PORT_REQUIRED_INERFACE		= "Port required interface";
			/**
Value to be used for Property parameter of addColumn method, 
when RelationAttributeFrom is selected for the Type parameter of addColumn method.
			**/
			public static final	 String		PROVIDED_INERFACE_OPERATIONS		= "Provided interface operations";
			/**
Value to be used for Property parameter of addColumn method, 
when RelationAttributeFrom is selected for the Type parameter of addColumn method.
			**/
			public static final	 String		REQUIRED_INERFACE_OPERATIONS		= "Required interface operations";
		}
		/**
Contains values to be used for Property parameter of addColumn method, 
when RequirementAttribute is selected for the Type parameter of addColumn method.

		**/
		public static final class RequirementAttribute
		{
			/**
Value to be used for Property parameter of addColumn method, 
when RequirementAttribute is selected for the Type parameter of addColumn method.
			**/
			public static final	 String		ID		= "ID";
			/**
Value to be used for Property parameter of addColumn method, 
when RequirementAttribute is selected for the Type parameter of addColumn method.
			**/
			public static final	 String		SPECIFICATION		= "Specification";
		}
		/**
Contains values to be used for Property parameter of addColumn method, 
when RelationAttributeTo is selected for the Type parameter of addColumn method.

		**/
		public static final class RelationAttributeTo
		{
			/**
Value to be used for Property parameter of addColumn method, 
when RelationAttributeTo is selected for the Type parameter of addColumn method.
			**/
			public static final	 String		VIA_PORT		= "Via port";
			/**
Value to be used for Property parameter of addColumn method, 
when RelationAttributeTo is selected for the Type parameter of addColumn method.

			**/
			public static final	 String		PORT_PROVIDED_INERFACE		= "Port provided interface";
			/**
Value to be used for Property parameter of addColumn method, 
when RelationAttributeTo is selected for the Type parameter of addColumn method.
			**/
			public static final	 String		TO_ELEMENT		= "To element";
			/**
Value to be used for Property parameter of addColumn method, 
when RelationAttributeTo is selected for the Type parameter of addColumn method.
			**/
			public static final	 String		PORT_REQUIRED_INERFACE		= "Port required interface";
			/**
Value to be used for Property parameter of addColumn method, 
when RelationAttributeTo is selected for the Type parameter of addColumn method.
			**/
			public static final	 String		PROVIDED_INERFACE_OPERATIONS		= "Provided interface operations";
			/**
Value to be used for Property parameter of addColumn method, 
when RelationAttributeTo is selected for the Type parameter of addColumn method.
			**/
			public static final	 String		REQUIRED_INERFACE_OPERATIONS		= "Required interface operations";
		}
	}
	/**
This class contains constant values for use with the methods setFromElementTypesUseQueryOrElementsList and setToElementTypesUseQueryOrElementsList.
	**/
	public static final class QueryOrElementsList
	{
		/**
When QUERY is used as the parameter for the methods setFromElementTypesUseQueryOrElementsList and setToElementTypesUseQueryOrElementsList, it indicates that a query is going to be used to determine the "from" element types or "to" element types for the relation table.

		**/
		public static final	 int		QUERY		= 1;
		/**
When ELEMENTS_LIST is used as the parameter for the methods setFromElementTypesUseQueryOrElementsList and setToElementTypesUseQueryOrElementsList, it indicates that elements selected in the element types list will be used to specify the "from" element types or "to" element types for the  relation table.
		**/
		public static final	 int		ELEMENTS_LIST		= 0;
	}

}

